//===----------- FernandoLiveStack.cpp - Printing prototype ---------------===//
//
//              Register Allocation via Coloring of Chordal Graphs 
//
// This file was developed by Fernando Magno Quintao Pereira, fernando at ucla
// dot edu, and it holds the variables alive at certain point during a scan of
// the control flow graph.
//
// Date: May 26th, 2006
//
//===----------------------------------------------------------------------===//
//
// This file can be used during a scan of the control flow graph to hold
// the values of variables alive at each point of the graph traversal.
// It is essentially a stack of live variables. Every time a variable is
// defined, it can be added to the stack. When it is killed, it can be
// removed. This file only handle virtual registers, because physical
// registers are not supposed to be alive across basic blocks.
//
//===----------------------------------------------------------------------===//

#ifndef LIVE_SET_FERNANDO_H
#define LIVE_SET_FERNANDO_H

namespace llvm {

  class Fernando_LiveSet {

    public:
      /// if this instruction defines a register, then marks the register as
      /// alive. If it uses registers, and the registers are not used
      /// thereafter, it marks the registers as dead.
      void update_live_ranges(MachineInstr *machine_inst);

      /// prints the set of live registers
      std::string to_string();

      /// constructor method.
      /// Param: lv - liveness information to be used to determine when
      /// temporaries are dead.
      Fernando_LiveSet(LiveVariables *lv) {
          this->live_ranges = lv;
      }

    private:
      /// live_variables - This list is a mapping from virtual register number
      /// to variable information. FirstVirtualRegister is subtracted from
      /// the virtual register number before indexing into this list.
      std::vector<bool> live_variables;

      /// Live_ranges - This is a mapping between variables and their live
      /// ranges. It is used so the dead variables can be removed from the
      /// live set.
      LiveVariables *live_ranges;

      /// This function scans all the operands of target instruction, and mark
      /// them as alive. Because the first operands are always definition, it
      /// will stop after the first use operand is found.
      void mark_defs_alive(MachineInstr *mi);

      /// This function scans all the operands in the target instruction that
      /// are uses, and mark them as dead. This function is used because the
      /// killing list in LiveVariables::VarInfo does not contain the uses of
      /// PHI functions, so, it should be called only for PHI instructions.
      void mark_uses_dead(MachineInstr *mi);

  };  // end of Fernando_LiveSet declaration

} // end llvm namespace

#endif
