//===----------------- NeatPrint_Fer.cpp - Layout output ------------------===//
//
//              Register Allocation via Coloring of Chordal Graphs 
//
// This class contains some handy functions to generate strings describing
// instructions, register, etc.
//
// Date: June 17th, 2006
//
//===----------------------------------------------------------------------===//
//
// This file contains functions that produce strings describing instructions,
// operands, and other data structures of the LLVM platform.
//
//===----------------------------------------------------------------------===//

namespace llvm {

#ifndef NEAT_PRINTER_FER
#define NEAT_PRINTER_FER
 
class NeatPrint_Fer {

public:

/// Returns a string describing the name of the instruction.
static std::string print_instr
                        (const MachineInstr & mi, const MachineFunction & mf) {
    const TargetMachine & tm = mf.getTarget();
    const TargetInstrInfo * tii = tm.getInstrInfo();
    const TargetInstrDescriptor & tid = tii->get(mi.getOpcode());
    std::ostringstream aux;
    aux << tid.Name;
    return aux.str();
}

/// Returns a string describing the name of the register.
static std::string print_op
                      (unsigned reg, const MachineFunction & mf) {

    std::ostringstream reg_name;
    if(MRegisterInfo::isVirtualRegister(reg)) {
        reg_name << "t" << (reg - MRegisterInfo::FirstVirtualRegister);
    } else {
        const TargetMachine & target_machine = mf.getTarget();
        const MRegisterInfo * reg_info = target_machine.getRegisterInfo();
        reg_name << reg_info->getName(reg);
    }
    return reg_name.str();
}

private:

};

#endif

}
